"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const _ = __importStar(require("underscore"));
const globals_1 = __importDefault(require("./globals"));
const logging_1 = __importDefault(require("./logging"));
const CONFIG_DEFAULTS = {
    filterPattern: "",
    normalizedFilterID: true,
    createLambdaPermission: true
};
class LogForwardingPlugin {
    constructor(serverless, options, v3Utils) {
        this.config = null;
        // The key of a lambda permission in CloudFormation resources
        this.permissionId = "LogForwardingLambdaPermission";
        this.serverless = serverless;
        this.options = options;
        this.provider = this.serverless.getProvider("aws");
        this.hooks = {
            "package:initialize": this.updateResources.bind(this)
        };
        globals_1.default.serverless = serverless;
        if (v3Utils === null || v3Utils === void 0 ? void 0 : v3Utils.log) {
            globals_1.default.v3Utils = v3Utils;
        }
        // schema for the function section of serverless.yml
        this.serverless.configSchemaHandler.defineFunctionProperties("aws", {
            properties: {
                logForwarding: {
                    type: "object",
                    properties: {
                        enabled: {
                            type: "boolean"
                        }
                    },
                    required: ["enabled"]
                }
            },
            required: []
        });
        // schema for the custom props section of serverless.yml
        this.serverless.configSchemaHandler.defineCustomProperties({
            properties: {
                logForwarding: {
                    type: "object",
                    properties: {
                        destinationARN: { type: "string" },
                        roleArn: { type: "string" },
                        filterPattern: { type: "string" },
                        normalizedFilterID: { type: "string" },
                        stages: {
                            type: "array",
                            uniqueItems: true,
                            items: { type: "string" }
                        },
                        createLambdaPermission: { type: "boolean" }
                    },
                    required: ["destinationARN"]
                }
            },
            required: ["logForwarding"]
        });
    }
    loadConfig() {
        const { service } = this.serverless;
        if (!service.custom || !service.custom.logForwarding) {
            throw new Error("Serverless-log-forwarding configuration not provided.");
        }
        this.config = Object.assign(Object.assign({}, CONFIG_DEFAULTS), service.custom.logForwarding);
        if (this.config.destinationARN === undefined) {
            throw new Error("Serverless-log-forwarding is not configured correctly. Please see README for proper setup.");
        }
    }
    updateResources() {
        this.loadConfig();
        const stage = this.getStage();
        if (this.config.stages && !this.config.stages.includes(stage)) {
            logging_1.default.logWarning(`Log Forwarding is ignored for ${stage} stage`);
            return;
        }
        logging_1.default.logInfo("Updating Log Forwarding Resources...");
        const resourceObj = this.createResourcesObj();
        const cfResources = this.getResources();
        _.extend(cfResources, resourceObj);
        logging_1.default.logInfo("Log Forwarding Resources Updated");
    }
    getResources() {
        const { service } = this.serverless;
        if (service.resources === undefined) {
            service.resources = {
                Resources: {}
            };
        }
        if (service.resources.Resources === undefined) {
            service.resources.Resources = {};
        }
        return service.resources.Resources;
    }
    createResourcesObj() {
        const { service } = this.serverless;
        const resourceObj = {};
        const createLambdaPermission = this.config.createLambdaPermission && !this.config.roleArn;
        if (createLambdaPermission) {
            const permission = this.makeLambdaPermission();
            resourceObj[this.permissionId] = permission;
        }
        _.keys(service.functions)
            .filter((functionName) => {
            const { logForwarding = {} } = this.serverless.service.getFunction(functionName);
            if (logForwarding.enabled === undefined) {
                return true;
            }
            return !!logForwarding.enabled;
        })
            .forEach((functionName) => {
            const filterId = this.getFilterId(functionName);
            const dependsOn = createLambdaPermission ? [this.permissionId] : [];
            const filter = this.makeSubsctiptionFilter(functionName, dependsOn);
            resourceObj[filterId] = filter;
        });
        return resourceObj;
    }
    getFilterId(functionName) {
        const filterName = this.config.normalizedFilterID
            ? this.provider.naming.getNormalizedFunctionName(functionName)
            : functionName;
        return `SubscriptionFilter${filterName}`;
    }
    getStage() {
        const { stage } = this.options;
        if (stage && stage !== "") {
            return stage;
        }
        return this.serverless.service.provider.stage;
    }
    makeSubsctiptionFilter(functionName, deps) {
        const functionObject = this.serverless.service.getFunction(functionName);
        const logGroupName = this.provider.naming.getLogGroupName(functionObject.name);
        const logGroupId = this.provider.naming.getLogGroupLogicalId(functionName);
        const roleObject = this.config.roleArn ? { RoleArn: this.config.roleArn } : {};
        return {
            Type: "AWS::Logs::SubscriptionFilter",
            Properties: Object.assign({ DestinationArn: this.config.destinationARN, FilterPattern: this.config.filterPattern, LogGroupName: logGroupName }, roleObject),
            DependsOn: _.union(deps, [logGroupId])
        };
    }
    makeLambdaPermission() {
        const { region } = this.serverless.service.provider;
        const principal = `logs.${region}.amazonaws.com`;
        return {
            Type: "AWS::Lambda::Permission",
            Properties: {
                FunctionName: this.config.destinationARN,
                Action: "lambda:InvokeFunction",
                Principal: principal
            }
        };
    }
}
module.exports = LogForwardingPlugin;
